@@ -1,5 +1,4 @@
 {-# LANGUAGE OverloadedStrings, FlexibleInstances #-}
-{-# LANGUAGE LambdaCase #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Documentation.Haddock.ParserSpec (main, spec) where
@@ -11,6 +10,8 @@ import           Documentation.Haddock.Doc (docAppend)
 import           Test.Hspec
 import           Test.QuickCheck
 
+import           Prelude hiding ((<>))
+
 infixr 6 <>
 (<>) :: Doc id -> Doc id -> Doc id
 (<>) = docAppend
@@ -57,7 +58,7 @@ spec = do
         "&#x65;" `shouldParseTo` "e"
 
       it "allows to backslash-escape characters except \\r" $ do
-        property $ \case
+        property $ \y -> case y of
           '\r' -> "\\\r" `shouldParseTo` DocString "\\"
           x -> ['\\', x] `shouldParseTo` DocString [x]
 
@@ -208,6 +209,10 @@ spec = do
       it "supports title for deprecated picture syntax" $ do
         "<<b a z>>" `shouldParseTo` image "b" "a z"
 
+    context "when parsing display math" $ do
+
+      it "accepts markdown syntax for display math containing newlines" $ do
+         "\\[\\pi\n\\pi\\]" `shouldParseTo` DocMathDisplay "\\pi\n\\pi"
 
     context "when parsing anchors" $ do
       it "parses a single word anchor" $ do
